<?php

namespace Drupal\beehotel_pricealterator;

use Drupal\Core\Session\AccountInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class Alter in charge of the price alteration.
 *
 * @package Drupal\beehotel_pricealterator\Services
 */
class Alter {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The tempstore factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * The preparing Class.
   *
   * @var \Drupal\beehotel_pricealterator\PreAlter
   */
  protected $preAlter;

  /**
   * The plugin manager.
   *
   * @var \Drupal\beehotel_pricealterator\PriceAlteratorPluginManager
   */
  protected $priceAlteratorPluginManager;

  /**
   * Alter constructor.
   *
   * @param \Drupal\Core\Session\AccountInterface $currentUser
   *   The current user.
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The tempstore factory.
   * @param \Drupal\beehotel_pricealterator\PreAlter $pre_alter
   *   The tempstore factory.
   * @param \Drupal\beehotel_pricealterator\PriceAlteratorPluginManager $price_alterator_plugin_manager
   *   The tempstore factory.
   */
  public function __construct(AccountInterface $currentUser, PrivateTempStoreFactory $temp_store_factory, PreAlter $pre_alter, PriceAlteratorPluginManager $price_alterator_plugin_manager) {
    $this->currentUser = $currentUser;
    $this->tempStoreFactory = $temp_store_factory;
    $this->preAlter = $pre_alter;
    $this->priceAlteratorPluginManager = $price_alterator_plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_user'),
      $container->get('tempstore.private'),
      $container->get('beehotel_pricealterator.prealter'),
      $container->get('plugin.manager.beehotel.pricealterator'),
    );
  }

  /**
   * Contains the alteration flow.
   */
  public function alter($data) {

    $tempstore = $this->tempStoreFactory->get('beehotel_pricealterator');

    // 1. Get content.
    // 1a. Base Table with base price.
    $basetable = $this->preAlter->baseTable($data);

    // 1b. Available Alterators.
    $alterators = $this->priceAlteratorPluginManager->sortedAlterators();

    // 1c. Enabled Alterators.
    $alterators = $this->checkStatus($alterators);
    $tempstore->set('beehotel_pricealterator_log', $alterators);

    /* 2. Loop per night.
     * Drupal Commerce calls Resolver once per order item (not once per night)
     * We loop nights here, providing an average price
     * to be multiplied by items (nights).
     * @todo keep track of every night price into the $data array.
     */
    for ($n = 0; $n < $data['norm']['dates_from_search_form']['days']; $n++) {
      $data['tmp'] = NULL;
      $data['tmp']['night_timestamp'] =
        $data['norm']['dates_from_search_form']['checkin']['timestamp'] + (24 * 3600 * $n);
      $data['tmp']['night_date'] = date('Y-m-d', $data['tmp']['night_timestamp']);

      foreach ($alterators as $a) {
        $alterator = new $a['class']([], [], [], []);
        $data = $alterator->alter($data, $basetable);
        if (isset($data['tmp']['price'])) {
          $price[$data['tmp']['night_timestamp']] = $data['tmp']['price'];
        }
      }
    }

    $price = array_filter($price);
    if (count($price)) {
      $average = array_sum($price) / count($price);
    }

    $data['amount'] = $average;
    return $data;
  }

  /**
   * Validate data.
   *
   * Check name change to lowerCamel format.
   */
  private function checkStatus($alterators) {

    $a = [];
    foreach ($alterators as $item) {
      if ($item['status'] == 1) {
        $a[] = $item;
      }
    }

    return $a;
  }

  /**
   * How to send this output to the debug block?
   *
   * @todo implement the feature.
   */
  public function beehotelLog($data, $context) {}

}
